// (c) 2006, 2007 Nerdfever.com
// For "Rev3" hardware (1st PCB board)

// Drives Hitachi HM55B compass (Parallax 29123)
// Uses SPI interface


#include <math.h>
#include "hardware.h"
#include "djllib.h"
#include "tweaks.h"
#include "compass.h"

// COMPASS INPUT (HM55B type, SPI interface)

void falling_edge(void)		// output a falling edge on compass clock line
{
	COMPASS_CLK = HIGH;
	COMPASS_CLK = LOW;
}

// output bits to compass (LSB first)
void compass_output(UINT8 bits, UINT8 numbits)
{
	COMPASS_DIR = 0;		// output

	do
	{
		COMPASS_OUT = bits & 1;
		falling_edge();
		bits = bits >> 1;
	}
	while (--numbits);
}

// read bits from compass (MSB first)
UINT16 compass_input(UINT8 numbits)
{
	UINT16 result = 0;

	COMPASS_DIR = 1;		// input
	wait(0.050);			// wait ~50 microseconds for output pin to discharge

	do
	{
		falling_edge();
		result = (result << 1) | COMPASS_IN;
	}
	while (--numbits);

	return result;
}

void compass_reset(void)
{
	COMPASS_EN_ = HIGH;
	COMPASS_EN_ = LOW;

	compass_output(0b0000, 4);

	COMPASS_EN_ = HIGH;
}

// starts measurement
void compass_start(void)
{
	COMPASS_EN_ = HIGH;
	COMPASS_EN_ = LOW;

	compass_output(0b0001, 4);

	COMPASS_EN_ = HIGH;
}

// returns 32 bit result or EOF if busy
// Return value is:
//	16 bits X in hi word,
//	16 bits Y in lo word
//  signed 2s complement, MSB first
UINT32 compass_read(void)
{
	UINT16 resultX, resultY;

	COMPASS_EN_ = HIGH;
	COMPASS_EN_ = LOW;

	compass_output(0b0011, 4);

	if (compass_input(4) != 0b1100)		// if busy
	{
		COMPASS_EN_ = HIGH;
		return EOF;
	}

	resultX = compass_input(11);
	resultY = compass_input(11);

	COMPASS_EN_ = HIGH;

	if (resultX &  0b00000000010000000000)		// if a negative 11-bit value
		resultX |= 0b11111111100000000000;		// then sign-extend to 16 bits

	if (resultY &  0b00000000010000000000)		// if a negative 11-bit value
		resultY |= 0b11111111100000000000;		// then sign-extend to 16 bits

	return (((UINT32) resultX) << 16) + resultY;
}

INT16 compass_rawToDegrees(INT32 compass_raw)
{
	INT16 compassX, compassY;
	double radians, degrees;

	compassX = WORD_H(compass_raw);
	compassY = WORD_L(compass_raw);

	radians = atan2(-compassY, compassX);		// ~~20 ms @ 2 MHz
	degrees = radians * (180.0/PI);

	return (INT16) degrees+0.5;
}

/*
// for HM55B / Parallax module #29123 compass module
// returns compass heading in degrees (-180 to +180)
INT16 compass_degrees(void)
{
	INT32 compass_raw;
	INT16 compassX, compassY;
	double radians, degrees;

	compass_start();

	do
	{
		// BE AWARE - if CLK or data glitches program will HANG here.  Better to poll and do a
		// COMPASS RESET if no response in reasonable period of time (40 ms = reasonable).
		compass_raw = compass_read();
	}
	while (compass_raw == EOF);

	return compass_rawToDegrees(compass_raw);
}
*/