// (c) 2006 Nerdfever.com

#include "djllib.h"
#include "nmeaGps.h"
#include "stateMachine.h"
#include "globals.h"

// GLOBALS

LOG_ABSOLUTE_GPS_RECORD FixA;
LOG_ABSOLUTE_GPS_RECORD FixB;
LOG_ABSOLUTE_GPS_RECORD *LastGPSFixPtr = &FixA;	// points at most recent GPS fix (fully decoded)
LOG_ABSOLUTE_GPS_RECORD *NextGPSFixPtr = &FixB;	// points at next GPS fix (being received now)
UINT8 AnyFix 		= FALSE;					// flag tells if we got a GPS fix that needs processing
UINT8 PositionFix 	= FALSE;					// flag tells if "AnyFix" is a valid 3D position fix
UINT8 ValidFixCount = 0;						// counts valid GPS fixes (255 max)
UINT8 FixCount 		= 0;						// counts valid GPS fixes (even invalid ones) (255 max)
LOG_ABSOLUTE_GPS_RECORD LastValidLoggedFix;		// holds most recent valid GPS fix that got logged

UINT8 DoLogStatus = 0;							// flag for whether to log status
UINT8 DoLogFix = 0;								// flag for whether to log latest GPS fix

double DeltaTC;									// slowTCpressure - fastTCpressure
double LaunchThreshold = LAUNCH_THRESHOLD;		// value of deltaTC that indicates launch

UINT16 volatile ServoPulseLengthUTicks;			// length of servo pulse, microticks
UINT32 volatile RTC = 0;						// real time clock - updated in ISR

UINT8 Buzzer 		= OFF;						// virtual noise buzzer on/off
UINT8 Horn   		= OFF;						// virtual loud horn (on/slow/fast/etc./off)
UINT8 Beep			= OFF;						// virutal beeper (response codes)

UINT16 SwRedHeld	= 0;						// # RTC ticks SW_RED held down continuously
UINT16 SwYellowHeld = 0;						// # RTC ticks SW_YELLOW held down continuously
UINT16 SwGreenHeld 	= 0;						// # RTC ticks SW_GREEN held down continuously

double SlowTCpressure;							// slow TC pressure value
double FastTCpressure;							// fast TC pressure value
double PadPressure;								// pressure at launch pad (ground level)
double ApogeePressure;							// pressure reading at apogee detection
UINT24 Pressure;								// last pressure value read from sensor
UINT32 CycleCount;								// counts of cycles thru main loop since reset

UINT8 LDet;										// launch detect flag
UINT8 ADet;										// apogee detect flag

UINT32 IRCode;									// IR camera message. Bits are sent right-to-left (1=IR on).

// state machine

UINT8 State = IDLE;								// state variable
UINT8 Substate = 0;								// substate variable

#pragma udata NMEA_Buffer

// these support the ISR uart receive buffer
char volatile NMEAbuffer[NMEA_LINES][NMEA_BUFSIZE];	// ISR buffer for rx'd sentances
UINT8 volatile NMEAwriteLine = 0;				// points at next line to write to
UINT8 NMEAreadLine = 0;							// points at next line to read from
UINT8 volatile NMEAlineCount = 0;				// number of complete lines in NMEA receive buffer
UINT8 volatile NMEAcharCount = 0;				// number of bytes written on current line
UINT32 LastUARTrxRTC = 0;						// lsbytes of RTC at reception of most recent UART byte
UINT32 GPSBurstStartRTC;						// RTC of start of most recent GPS data burst

#pragma udata Flash_Buffer

// these support the "safe" writing to flash (avoid missing interrpts during flash acess CPU stall)
// Hardware related globals

UINT24 FlashWritePtr = FLASH_BASE;	// next byte of flash memory to be written (virtual)
UINT24 FlashFlushPtr = FLASH_BASE;	// flash address for next buffer flush to flash memory
UINT8 FlashBuffer[FLASH_BUFFER_ROWS][FLASH_ROW_SIZE];
UINT8 FlashBufferWriteRow = 0;		// which row buffer we are now writing to
UINT8 FlashBufferWriteCount = 0;	// number of bytes in current buffer
UINT8 FlashBufferRowCount = 0;		// number of filled-up buffers waiting to be written to flash
UINT8 FlashBufferFlushRow = 0;		// which row buffer we will flush from next

#pragma udata

UINT8 Dummy;									// dummy value to replace non-existent outputs

void _dummyFunction()							// it seems C18 doesn't like .c files with no functions in them
{}