// (c) 2006, 2007 Nerdfever.com
// For "Rev3" hardware (1st PCB board)

// PRAGMAS

#pragma config MCLRE 	= OFF		// disable master clear (pin 1 used as input)
#pragma config OSC 		= INTIO67	// internal oscillator, OSC1 is RA7, OSC2 is RA6
#pragma config WDT 		= OFF		// turn off watchdog timer
#pragma config LVP 		= OFF 		// diable low voltage programming (ignore noise on pin 26)
#pragma config STVREN 	= ON		// for some reason this needs to be set either ON or OFF or no go HW
#pragma config PBADEN 	= OFF		// set PORTB pins as digital I/O on Reset

// INCLUDES

#include <string.h>
#include "hardware.h"
#include "djllib.h"
#include "globals.h"

// UART

// always does 8,N,1
void SetSerial(UINT32 baudrate)
{
	SPBRGH = BAUD_VALUE(baudrate) >> 8;		// high byte of BRG
	SPBRG = BAUD_VALUE(baudrate) & 0xff;	// low byte of BRG
	BAUDCON = 0b01001000;					// setup 16 bit BRG
	TXSTA = 0b00100110;						// async, 8 bits, no break, hi speed BRG (Fosc/4)
	RCSTA = 0b10010000;						// enable port, enable rx
}

// writes one byte to UART
void WriteByte(UINT8 byte)
{
	while (!PIR1bits.TXIF)		// wait for TX buffer to empty
		;

	TXREG = byte;				// write it
}

// SERVO

// set servo postition, range 0 to 255
void SetServoPos(UINT8 position)
{
	UINT16 u_ticks = MS_TO_UTICKS(RIGHT) + ((MS_TO_UTICKS(LEFT - RIGHT) * position) >> 8);

	DI();
	ServoPulseLengthUTicks = u_ticks;
	EI();
}

// write a single block of FLASH_ROW_SIZE bytes from RAM to internal FLASH (program memory)
// note that FLASHaddr MUST be exactly on a FLASH_ROW_SIZE boundary
void WriteFlashBlock(UINT8 *RAMptr, UINT24 FLASHaddr)
{
	UINT8 i;

	WRITE_LED = !WRITE_LED;			// flash LED as diagnostic

	// erase block in FLASH

	TBLPTR = FLASHaddr;				// flash address to erase
	EECON1bits.EEPGD = 1;
	EECON1bits.CFGS = 0;
	EECON1bits.WREN = 1;
	EECON1bits.FREE = 1;

	DI();
	EECON2 = 0x55;
	EECON2 = 0xAA;
	EECON1bits.WR = 1;				// start erase; note that CPU will stall for 2 ms here
	EI();

	EECON1bits.WREN = 0;			// turn write enable off

	// copy block to table latches

	for(i=0; i<FLASH_ROW_SIZE; i++)
	{
 		TABLAT = *RAMptr++;	// put a char into the table latch register

	 	_asm
  		TBLWTPOSTINC     			// write to holding register and post-increment TBLPTR
 		_endasm
	}

	// do the write to flash

	TBLPTR = FLASHaddr;				// force to "within intended range"
	EECON1bits.EEPGD = 1; 			// write the holding registers to flash
	EECON1bits.CFGS = 0;
	EECON1bits.WREN = 1;
	EECON1bits.FREE = 0;

	DI();
	EECON2 = 0x55;
	EECON2 = 0xAA;
	EECON1bits.WR = 1;				// per 6.5.1/PIC16F2620 datashet, CPU STALLS here for ~ 2 milliseconds!
	EI();

	EECON1bits.WREN = 0;			// turn write enable off

	WRITE_LED = !WRITE_LED;			// flash LED as diagnostic
}

/* The following work OK but are commented out to save space


// EEPROM AND FLASH

// read bytes from internal EEPROM to RAM
void ReadEEPROM(INT8 *RAMptr, INT16 EEPROMaddr, INT16 count)
{
	while (count--)
	{
		EECON1bits.EEPGD = 0;			// set for EEPROM access

//		EEADR = EEPROMaddr;
		EEADRH = EEPROMaddr >> 8;		// high bits of EEPROM address
		EEADR = EEPROMaddr++ & 0xff;	// low 8 bits of EEPROM address

		EECON1bits.CFGS = 0; 			// access data registers

		EECON1bits.RD = 1;				// read into EEDATA

		*RAMptr++ = EEDATA;				// read byte
	}
}


// write bytes from RAM to internal EEPROM
void WriteEEPROM(INT8 *RAMptr, INT16 EEPROMaddr, INT16 count)
{
	EECON1bits.EEPGD = 0;				// access EEPROM
	EECON1bits.CFGS = 0; 				// access data registers
	EECON1bits.WREN = 1;				// write enable

	while (count--)
	{
		EEADRH = EEPROMaddr >> 8;		// high bits of EEPROM address
		EEADR = EEPROMaddr++ & 0xff;	// low 8 bits of EEPROM address

		EEDATA = *RAMptr++;				// byte to write

		_asm
			BCF	  INTCON, 7, ACCESS 	// disable interrupts
			MOVLW 0x55
			MOVWF EECON2,0
			MOVLW 0xAA
			MOVWF EECON2,0				// magic 0x55, 0xAA write sequence
			BSF	  EECON1, 1, ACCESS		// set WR bit; start write
			NOP							// seems to help; don't know why
			BSF	  INTCON, 7, ACCESS		// enable interrupts
		_endasm

//This is the C equivalant of the assembly above - for some strange reason
//   the assembly code appears to be more reliable.  It's probably quicker.
//
//		DI();							// disable interrupts
//		EECON2 = 0x55;
//		EECON2 = 0xAA;					// magic 0x55, 0xAA write sequence
//		EECON1bits.WR = 1;				// initiate write
//		EI();							// enable interrupts

		while (EECON1bits.WR)			// wait for write to complete (~ 4 ms per byte = 2000 bps)
			;
	}

	PIR2bits.EEIF = 0;					// reset EEIF
	EECON1bits.WREN = 0;				// disable writes
}

// returns EOF if verify is OK, offset of first failure if not OK
INT16 VerifyEEPROM(INT8 *RAMptr, INT16 EEPROMaddr, INT16 count)
{
	UINT16 i;
	UINT8 b;

	for (i=0; i < count; i++)
	{
		ReadEEPROM((INT8*)&b, EEPROMaddr+i, 1);	// read 1 byte

		if (b != RAMptr[i])						// verify
			return i;
	}

	return EOF;
}


// read bytes from internal FLASH (program memory) to RAM
void ReadFlash(void *RAMptr, UINT24 FLASHaddr, UINT24 count)
{
	EECON1bits.EEPGD = 1;			// set for FLASH access
	TBLPTR = FLASHaddr;

	while (count--)
	{
		_asm
		TBLRDPOSTINC				// read byte into TABLAT
		_endasm

		*(char*)RAMptr++ = TABLAT;	// read byte
	}
}

//	This version allows random-access writes and avoids RAM buffering by
//  reading back partially-filled flash blocks before writing new data.
//  It ensures that all data is in fact written to flash at exit (not
// 	buffered in RAM), but at the expense of being slow (reading flash is slow).

// write bytes from RAM to internal FLASH (program memory)
void WriteFlash(void*RAMptr, UINT24 FLASHaddr, UINT24 count)
{
	UINT8 buffer[FLASH_ROW_SIZE];
	UINT24 dup1_count, dup2_count, write_count;
	UINT24 ROMblock_ptr = FLASHaddr & (~(FLASH_ROW_SIZE-1));

	do	// for each block of FLASH_ROW_SIZE
	{
		dup1_count = max(FLASHaddr - ROMblock_ptr, 0);
		dup2_count = max(FLASH_ROW_SIZE - count - dup1_count, 0);
		write_count = FLASH_ROW_SIZE - dup1_count - dup2_count;

		// build buffer
		ReadFlash(buffer, ROMblock_ptr, dup1_count);
		memcpy(buffer + dup1_count, (const void *)RAMptr, write_count);
		ReadFlash(buffer + dup1_count + write_count, ROMblock_ptr + dup1_count + write_count, dup2_count);

		WriteFlashBlock(buffer, ROMblock_ptr);

		count -= write_count;
		ROMblock_ptr += FLASH_ROW_SIZE;
		FLASHaddr += write_count;
		RAMptr += write_count;
	}
	while (count);
}


//	This version writes serially (streaming mode) to flash, and uses RAM
//  buffering to go faster.  In the event of a crash or hardware failure,
//  anything still in the RAM buffer is lost.
//
//	It writes at the address of the global FlashPtr varaible, which is
//	updated by this routine.  It does NOT check that there is enough
//	room for the data - check that yourself before calling this.
//
//	FLASH_ROW_SIZE _must_ be a power of 2 for this to work.


// write bytes from RAM to internal FLASH (program memory)
void WriteFlashStreaming(void *RAMptr, UINT24 bytes)
{
	static UINT8 buffer[FLASH_ROW_SIZE];
	UINT16 num;
	UINT16 buffer_index = FlashPtr & (FLASH_ROW_SIZE-1);	// low order bits of FlashPtr;

	while(1)
	{
		// copy as much as will fit in buffer (or till end)

		num = min(bytes, FLASH_ROW_SIZE - buffer_index);	// number of bytes to copy
		memcpy((void*) &buffer+buffer_index, (void*) RAMptr, num);

		bytes -= num;
		FlashPtr += num;
		buffer_index = FlashPtr & (FLASH_ROW_SIZE-1);

		// if buffer full, flush it to flash

		if (!buffer_index)
			WriteFlashBlock(buffer, FlashPtr - FLASH_ROW_SIZE);

		if (!bytes)
			break;											// done

		// more to do, loop back.

		RAMptr += num;
	}
}

*/ // end of commented out routines

/*	This version writes serially (streaming mode) to flash, and counts
	on companion routine FlushFlashBufferSafe() to actually do the flash write
	at a time when it's "safe" to do so.  ("Safe" means that no high-
	speed interrupts are expected that would be missed during the
	flash write stall.)

	In the event of a crash or hardware failure,anything still in the
	RAM buffer is lost.

	It writes at the address of the global FlashPtr varaible, which is
	updated by this routine.  It does NOT check that there is enough
	room for the data - check that yourself before calling this.
*/

// write bytes from RAM to internal FLASH (program memory) indirectly via FlushFlashBufferSafe()
void WriteToFlashBuffer(void *RAMptr, UINT24 bytes)
{
	UINT16 num;

	while(1)
	{
		// copy as much as will fit in current buffer (or till end)

		num = min(bytes, FLASH_ROW_SIZE - FlashBufferWriteCount);	// number of bytes to copy
		memcpy((void*) &FlashBuffer[FlashBufferWriteRow]+FlashBufferWriteCount, (void*) RAMptr, num);

		bytes -= num;
		FlashBufferWriteCount += num;
		FlashWritePtr += num;

		if (FlashBufferWriteCount == FLASH_ROW_SIZE)				// if buffer is full
		{
			FlashBufferWriteCount = 0;
			FlashBufferWriteRow = ++FlashBufferWriteRow % FLASH_BUFFER_ROWS;
			FlashBufferRowCount++;
		}

		if (!bytes)
			break;													// done

		// more to do, loop back.

		RAMptr += num;
	}
}

// Returns a flag.
// If TRUE, we are currently in-between GPS fixes and will continue to be
//	for at least "rtcTicks" more.
// If FALSE, we are either in a GPS fix burst, or one is expected anytime.
// This is useful for avoiding UART overrun errors caused by GPS transmission
// during flash write stall.  GPS data burst is up to GPS_BURST_PERIOD in
// duration (GPRMC + GPGGA, at 38400 bps takes 40 to 60 ms).
UINT8 BetweenGPSFixesFor(UINT8 rtcTicks)
{
	UINT32 ticksSinceLastUARTrx = RTC - LastUARTrxRTC;

	if (ticksSinceLastUARTrx < 2)	// 2 ticks because 1 tick can happen any time, 2 means UART burst is over
		return FALSE;				// we are currently in a GPS burst (or one ended so recently we can't tell the difference)

	if (between(ticksSinceLastUARTrx, GPS_FIX_INTERVAL-GPS_BURST_PERIOD-1-rtcTicks, GPS_FIX_INTERVAL))
		return FALSE;				// a GPS burst is expected any moment
	else
		return TRUE;
}

void FlushFlashBufferSafe(UINT8 maxWrites)
{
	while(FlashBufferRowCount && maxWrites-- && BetweenGPSFixesFor(MS(10)))
	{
		WriteFlashBlock((UINT8*) &FlashBuffer[FlashBufferFlushRow], FlashFlushPtr);

		FlashBufferRowCount--;
		FlashBufferFlushRow = ++FlashBufferFlushRow % FLASH_BUFFER_ROWS;
		FlashFlushPtr += FLASH_ROW_SIZE;
	}
}

// moves FlashPtr to a new write address and tosses any unwritten content of flash buffers
void ResetFlashPtrSafe(UINT24 flashAddr)
{
	FlashWritePtr = flashAddr;
	FlashFlushPtr = FlashWritePtr;

	FlashBufferWriteRow = 0;
	FlashBufferWriteCount = 0;
	FlashBufferRowCount = 0;
	FlashBufferFlushRow = 0;
}

// ADC INPUT (PRESSURE SENSOR)

/*

// starts a single ADC sample on channel as already setup and turned on
void StartADC(void)
{
 	ADCON0bits.GO = 1;
}

// polls if ADC conversion is done; returns EOF if not done, result if done
UINT16 PollADC((void)
{
	if (ADCON0bits.GO == FALSE)			// if ADC done
		return ADRES;
	else
		return EOF;
}

*/
UINT24 AccumulateADCsamples(UINT16 times)
{
	UINT24 accum = 0;

	while (times--)
	{
 		ADCON0bits.GO = 1;				// start conversion

		while (ADCON0bits.GO == TRUE)
			;

		accum += ADRES;

		// Tacq goes here, but is not needed at up to FOSC = 8 MHz
	}

	return accum;
}

// CAMERA

void TakePicture(void)
{
	IRCode = (SHUTTER_IR_CODE)*2;		// bits will be sent right-to-left
}

// DELAY LOOPS

// use wait() instead; this is for #define wait() use ONLY
void __waitloop_internal (UINT32 i)
{
	while (i--);
}

// GENERAL HARDWARE INIT

void SetupHardware(void)
{
	// set CPU frequency (default is 1 MHz on reset)

	OSCCON = (OSCCON & 0b10001111) | OSCVAL;

	// enable interrupts

	INTCON 	= 0b11000000;			// enable global & peripheral interrupts
	PIE1 	= 0b00100001;			// enable UART RX & Timer 1 overflow interrupts

	/*
		Notes on Timer1 operation to drive servo:

		Timer1 counts UP, starting from TMR_START.  It's a 16-bit counter, so it overflows
		when it reaches 65536.  It is set to generate an interrupt on overflow.

		In the ISR, we raise the leading edge of the pulse (start the pulse), and reset
		Timer1 to TMR_START.  Then we set the CCP module to lower the pin (end the pulse)
		when the timer reaches ccp_end.

		So the pulse interval is (65536 - TMR_START) ticks.

		The pulse duration is (ccp_end - TMR_START) ticks.

	*/

	// setup Timer 1 to generate periodic interrupts

	TMR1H 	= TMR_START >> 8;		// high byte of Timer1
	TMR1L 	= TMR_START & 0xff;		// low byte of Timer1
	T1CON 	= 0b10110001;			// 16 bit mode, int osc, 1:8 prescale, disable T1 osc, -, use internal FOSC/4, enable timer

	// setup Timer 3 to run at same rate as Timer 1

	T3CON	= 0b11110001;			// 16 bit mode, T3 to CCPs, 1:8 prescale, internal FOSC/4 clock, enable

	// initialize I/O port direction

	PORTA 	= 0b00000000;			// turn off all PORTA output bits
	TRISA 	= PORTA_IO;				// set PORTA IO

	PORTB 	= 0b00000000;			// turn off all PORTB output bits
	TRISB 	= PORTB_IO;				// set PORTB IO
	INTCON2bits.RBPU = 0;			// turn on weak pullups on PORTB

	PORTC 	= 0b00000000;			// turn off all PORTC output bits
	TRISC 	= PORTC_IO;				// set PORTC IO

	// Setup onboard comparator to level-convert GPS RX serial data
	// Note: GlobalSat EM-406 GPS is speced for 0 to 2.85vdc signal output levels

	TRISAbits.TRISA2 = 0; 			// set RA2/AN2/CVref as Output
	CVRCON = 0b11100111;			// CVref = 7/24 * Vdd = 1.458 volts @ 5v Vdd

	TRISAbits.TRISA0 = 1;			// RA0/AN0 is input
	TRISAbits.TRISA3 = 1;			// RA3/AN3 is input
	TRISAbits.TRISA4 = 0;			// set RA4/C1OUT as Output
	CMCON = 0b00010001;				// Comparator mode 001 (AN3 > AN0 >>> C1OUT*), inverted output

	// Setup ADC to read AN1

	ADCON1 = 0b00001101;			// Vref- = Vss, Vref+ = Vdd, set AN0 and AN1 as analog inputs
	ADCON0 = 0b00000100;			// Select AN1, keep ADC disabled
	ADCON2 = 0b10000101;			// right justified result, no auto aq time delay, use Fosc/16 clock
	ADCON0bits.ADON = 1;			// turn on ADC

	// set TMR2 and CCP2 into PWM mode for IR "on" pulse modulation
	PR2 = IR_MODULATION_PERIOD_UTICKS;
	CCPR2L = IR_MODULATION_PERIOD_UTICKS*IR_MODULATION_DUTY_CYCLE;	// duty cycle (square wave, or close enough)
	T2CON = 0b00000100;				// No postscale, Timer2 on, no prescale (increments once per 4 FOSC cycles)
	CCP2CON = 0b00000000;			// disable PWM (for now)
	SHUTTER = 0;					// turn OFF infrared output when not modulating

	// Setup compass

#ifdef COMPASS
	compass_reset();
	compass_start();				// start first measurement
#endif
}