// (c) 2006 Nerdfever.com

#include "hardware.h"
#include "globals.h"
#include "debug.h"
#include "peripherals.h"


// FUNCTIONS

// interrupt vector setup - must name service routine "isr()" (add POSTINC1 to nosave list when C18 supports it in future)
#pragma interrupt isr nosave=TBLPTR, TBLPTRU, TABLAT, PCLATH, PCLATU, section(".tmpdata"), section("MATH_DATA")

#pragma code int_vector=0x0008

void int_vector (void)
{
	_asm
	goto isr
	_endasm
}

#pragma code

// ISR

void isr(void)
{
	UINT8 timer1_expired;				// boolean flag

	// NOTE: Modify +n in TMR_START definition if the code between here and resetting TMR1L changes!!!

	PUSH_DEBUG_STATE();
	SET_DEBUG_STATE(CPU_BUSY);

	if (PIR1bits.TMR1IF)				// if Timer1 overflowed
	{
		timer1_expired = TRUE;			// this lets us get to UART RX quicker

		_asm NOP NOP NOP NOP _endasm	// round up interrupt period to a whole number of timer ticks

		// reset RTC timer for next interrupt

		TMR1H = TMR_START >> 8;			// high byte of Timer1
		TMR1L = TMR_START & 0xff;		// low byte of Timer1
	}
	else
		timer1_expired = FALSE;

	while (PIR1bits.RCIF)				// if UART RX buffer is full
	{
		// service UART

		char byte = RCREG; 				// read received byte from UART

		if (RTC - LastUARTrxRTC > GPS_BURST_PERIOD_MS/RTC_INTERRUPT_INTERVAL_MS)
			GPSBurstStartRTC = RTC;

		LastUARTrxRTC = RTC;			// remember tick of most recent RX byte

		if (byte == ',')
			byte = 0;					// substitute sequence of strings for NMEA CSV

		NMEAbuffer[NMEAwriteLine][NMEAcharCount++] = byte;

		if (RCSTAbits.OERR)				// if we got an overrun
		{
			RCSTAbits.CREN = 0;			// reset overrun by clearing CREN
			RCSTAbits.CREN = 1;			// then re-enabling it

			NMEAcharCount = 0;			// forget whole line so far (don't want an error)
			byte = 0;					// anything that isn't LF will do
		}

		if ((NMEAcharCount == NMEA_BUFSIZE-1) || (byte == LF))			// start new line?
		{
			NMEAwriteLine = (NMEAwriteLine + 1) & (NMEA_LINES - 1); 	// mod NMEA_LINES
			NMEAlineCount++;
			NMEAcharCount = 0;
		}
	}

	if (timer1_expired)
	{
		static UINT8 ticks_till_servo_pulse;

		ServiceBeep();

		RTC++;										// increment real-time clock

		if (!--ticks_till_servo_pulse)				// is it time to start a new servo pulse?
		{
			ticks_till_servo_pulse = SERVO_PULSE_INTERVAL / RTC_INTERRUPT_INTERVAL_MS;

			T3CONbits.TMR3ON = OFF;					// turn off Timer3

			TMR3H = 0;								// reset timer to zero
			TMR3L = 0;

			// setup CCP1 to end servo pulse after proper duration

			CCPR1H = ServoPulseLengthUTicks >> 8;
			CCPR1L = ServoPulseLengthUTicks & 0xff;

			// raise CCP high (start servo pulse)

			CCP1CON = 0b00001001;					// compare mode, init pin high, force low on match

			T3CONbits.TMR3ON = ON;					// start Timer3
		}

		// clock out next bit of IR code (if any)

		if (IRCode & 1)
			IR_ON;
		else
			IR_OFF;

		IRCode /= 2;								// shift message right 1 bit

		ServiceBeep();

		// done with Timer1 servicing

		PIR1bits.TMR1IF = 0;						// reset Timer1 overflow flag
	}

	POP_DEBUG_STATE();
}

