// (c) 2006, 2007 Nerdfever.com
// Data logging stuff

#include "logging.h"
#include <string.h>
#include "nmeaGps.h"
#include "globals.h"

// log GPS fixes

/*
void process_fix(FIX *fixptr)
{
	static UINT8 ejected = FALSE;						// TRUE = ejected, FALSE = didn't yet
	static UINT16 eeprom_bytes_left
		= EEPROM_SIZE - PRE_EJECT * sizeof(*fixptr);	// number of bytes written to EEPROM since ejection
	static UINT16 flash_bytes_left
		= FLASH_SIZE - PRE_EJECT * sizeof(*fixptr);		// number of bytes written to FLASH since ejection

	static UINT16 eeprom_writeptr = 0;
	static UINT16 flash_writeptr = 0;

	if (RTC < MILLISECONDS(10000))	// if recently started or reset
		return;

	if (LIGHT)						// do upon detection of ejection
	{
		ejected = TRUE;				// this is to remember, in case we see dark again later
		BUZZER = OFF;
	}

	fixptr->eject_flag = ejected;

	// here we start subtracting bytes ONLY after ejection, so that we
	// preserve the post-ejection data (don't overwrite it)

	if (ejected)
	{
		if (flash_bytes_left < sizeof(*fixptr))
		{
			BUZZER = ON;
			return;								// bail if no room left in FLASH
		}

		flash_bytes_left -= sizeof(*fixptr);	// account for fix we are about to write
	}

	// we either haven't ejected or we have and there is still flash space left - save the fix

	WriteFlash(fixptr, flash_writeptr + FLASH_BASE, sizeof(*fixptr));
	flash_writeptr = (flash_writeptr + sizeof(*fixptr)) & (FLASH_SIZE-1);

	// now do the same for the EEPROM - note that we test this after the FLASH because
	// there is less EEPROM space than FLASH (we run out of EEPROM first)

	if (ejected)
	{
		if (eeprom_bytes_left < sizeof(*fixptr))
		{
			BUZZER = ON;
			return;								// bail if no room left in EEPROM
		}

		eeprom_bytes_left -= sizeof(*fixptr);	// account for fix we are about to write
	}

	WriteEEPROM((INT8*)fixptr, eeprom_writeptr, sizeof(*fixptr));
	eeprom_writeptr = (eeprom_writeptr + sizeof(*fixptr)) & (EEPROM_SIZE-1);
}

*/

#pragma udata lag_buffer							// force this into a seperate udata section (it's "big")
LOG_STATUS_RECORD laggedWriteBuffer[LAGGED_RECORDS];// buffer for lagged write of log entries upon entering FLIGHT
#pragma udata

UINT8 laggedIndex = 0;								// index into laggedWriteBuffer
UINT8 laggedCount = 0;								// counts unflushed entries in buffer

void LogStatusToLaggedBuffer(LOG_STATUS_RECORD* status)
{
	laggedWriteBuffer[laggedIndex] = *status;			// copy entry into lagged buffer
	laggedIndex = ++laggedIndex % LAGGED_RECORDS;		// next index
	laggedCount = min(++laggedCount, LAGGED_RECORDS);	// increment count
}

// flush lagged records to Flash RAM
void FlushLaggedRecords(void)
{
	laggedIndex = ((laggedIndex + LAGGED_RECORDS)-laggedCount) % LAGGED_RECORDS;

	while(laggedCount)
	{
		WriteToFlashBuffer(&laggedWriteBuffer[laggedIndex], sizeof(LOG_STATUS_RECORD));
		laggedIndex = ++laggedIndex % LAGGED_RECORDS;
		laggedCount--;
	}
}

// writes init record to flash
void logInitRecord(void)
{
	LOG_INIT_RECORD init;

	init.spareBits			= 0;
	init.recordType			= LOG_INIT_RECORD_CODE;
	init.cycleCount8		= CycleCount;
	init.rtc16				= RTC16;
	init.flashSize 			= FLASH_SIZE;
	init.VREFmV				= Vdd * 1000;
	init.rtcTickIntervaluS	= RTC_INTERRUPT_INTERVAL_MS * 1000;
	init.gpsType			= GPS_TYPE;
	init.gpsFixIntervalmS	= (UINT16) GPS_FIX_INTERVAL * RTC_INTERRUPT_INTERVAL_MS;

	WriteToFlashBuffer(&init, sizeof(init));
}

void logAbsoluteGPSRecord(LOG_ABSOLUTE_GPS_RECORD *fixptr)
{
	fixptr->recordType = LOG_ABSOLUTE_GPS_RECORD_CODE;
	fixptr->spareBits = 0;
	fixptr->cycleCount8 = CycleCount;

	WriteToFlashBuffer(fixptr, sizeof(*fixptr));
}

void logDifferentialGPSRecord(LOG_ABSOLUTE_GPS_RECORD *fixptr)
{
	LOG_DIFFERENTIAL_GPS_RECORD diff;

	diff.recordType = LOG_DIFFERENTIAL_GPS_RECORD_CODE;
	diff.spareBits = 0;
	diff.cycleCount8 = CycleCount;

	AbsoluteToDifferentialGPSFix(&fixptr->fix, &diff.diffGPS);

	WriteToFlashBuffer(&diff, sizeof(diff));
}

void LogGPSRecord(LOG_ABSOLUTE_GPS_RECORD *fixptr)
{
	// determine whether to log an absolute or differential record

//	if (PositionFix && LastValidLoggedFix.fix.valid)
//		logDifferentialGPSRecord(LastGPSFixPtr);
//	else
		logAbsoluteGPSRecord(LastGPSFixPtr);

	if (PositionFix)
		LastValidLoggedFix = *LastGPSFixPtr;	// if this was a good fix (and logged), save it for reference
}

void InitializeLog(void)
{
	ResetFlashPtrSafe(FLASH_BASE);
	logInitRecord();						// write init record to flash
	LastValidLoggedFix.fix.valid = FALSE;	// force at least one absolute fix record
}