// (c) 2007 Nerdfever.com

#ifndef __LOGGING_H
#define __LOGGING_H

#include "djllib.h"
#include "nmeaGps.h"
#include "tweaks.h"

//#define PRE_EJECT		(16)			// number of seconds of pre-ejection data to preserve

#define LOG_INIT_RECORD_CODE				(1)
#define LOG_STATUS_RECORD_CODE				(2)
#define LOG_ABSOLUTE_GPS_RECORD_CODE		(3)
#define LOG_DIFFERENTIAL_GPS_RECORD_CODE	(4)

typedef struct	// init record (once per flight)
{
	unsigned	recordType 	: 4;	// Least Significant Bits
	unsigned	spareBits  	: 4;
	UINT8		cycleCount8;		// main.c "CycleCount" (8 LSB)
	UINT16		rtc16;				// 2 bytes, LSBs of RTC
	UINT24		flashSize;			// size of flash log record area, bytes
	UINT16		VREFmV;				// supply voltage for pressure sensor, millivolts
	UINT24		rtcTickIntervaluS;	// rtc tick interval, microseconds
	UINT8		gpsType;			// GPS type (make, model)
	UINT16		gpsFixIntervalmS;	// intended GPS fix interval, milliseconds
} LOG_INIT_RECORD;

typedef struct 	// log status record
{
	unsigned	recordType 	: 4;	// Least Significant Bits
	unsigned	swArmed 	: 1;	// safe/armed switch state (hi=armed)
	unsigned	ejDetect	: 1;	// 1 = ejection detected (via sensor)
	unsigned	ejCont	 	: 1;	// ejection continuity good (1=good)
	unsigned	lagged 	 	: 1;	// lagged record flag
	UINT8		cycleCount8;		// main.c "CycleCount" (8 LSB)
	UINT16		rtc16;				// 2 bytes, LSBs of RTC
	UINT16		rawPressure;		// 2 bytes
	unsigned	substate 	: 4;	// Substate (LSN)
	unsigned 	state    	: 4;	// State (MSN)
} LOG_STATUS_RECORD;

typedef struct	// absolute GPS record (used for 1st fix, 1st valid fix)
{
	unsigned	recordType	 : 4;	// Least Significant Bits
	unsigned	spareBits 	 : 4;
	UINT8		cycleCount8;		// main.c "CycleCount" (8 LSB)
	GPSFIX		fix;
} LOG_ABSOLUTE_GPS_RECORD;

typedef struct
{
	unsigned	recordType 	: 4;	// Least Significant Bits
	unsigned	spareBits  	: 4;
	UINT8		cycleCount8;		// main.c "CycleCount" (8 LSB)
	GPSFIX_D	diffGPS;
} LOG_DIFFERENTIAL_GPS_RECORD;

void FlushLaggedRecords(void);
void LogStatusToLaggedBuffer(LOG_STATUS_RECORD *logEntry);
void LogGPSRecord(LOG_ABSOLUTE_GPS_RECORD *fixptr);
void InitializeLog(void);

#endif // __LOGGING_H
