// Autopilot routine for PIC18LF2620, (c) 2006-2007 Nerdfever.com

#include "djllib.h"
#include "globals.h"
#include "peripherals.h"
#include "logging.h"
#include "stateMachine.h"
#include "debug.h"
#include "tweaks.h"

UINT8  nextRTC8;									// holds future rtc value at which to run thru main loop again
UINT32 rawCompass;									// raw compasss X and Y output
UINT16 rtcPressure;
LOG_STATUS_RECORD status;


void main(void)
{
	// init hardware

	SetupHardware();							// setup CPU speed, etc.

	LED_YELLOW = LED_GREEN = LED_RED = ON;		// flash LEDs on wakeup

	SetServoPos(127);							// middle position

	// init pressure average values

	SlowTCpressure = ReadPressureSensor();
	FastTCpressure = SlowTCpressure;

	// setup GPS

	InitGPS();

	// init complete, setup for main loop

	CycleCount = 0;

	InitStateMachine();

	nextRTC8 = RTC8 + LOOP_PERIOD;

	// main loop

	while (1)
	{
		// LOOP_PERIOD poll stuff

		CycleCount++;

		CheckForNewGPSFix();					// AnyFix true means we just got a NEW fix into GpsFix

		if (AnyFix)
			GPS_LED = !GPS_LED;					// invert flash on GPS fix

		rtcPressure = RTC16;
		Pressure = ReadPressureSensor();

		SlowTCpressure = SlowTCpressure + (Pressure - SlowTCpressure)/min(CycleCount, SLOW_TC);	// leaky bucket integrator
		FastTCpressure = FastTCpressure + (Pressure - FastTCpressure)/min(CycleCount, FAST_TC);	// leaky bucket integrator

		DeltaTC = SlowTCpressure - FastTCpressure;

		LDet = DeltaTC > LaunchThreshold;		// launch detect

		ADet = FastTCpressure <= Pressure;		// apogee detect (valid only in FLIGHT)

		PollSwitches();							// check which (if any) switches held down continuously

		if (AnyFix)
			GPS_LED = !GPS_LED;					// restore LED state

		// run the state machine once

		StateMachine();							// execute state machine

		// status logging (for lagged buffer even if not time to write to flash)

		status.recordType	= LOG_STATUS_RECORD_CODE;
		status.swArmed 		= SW_ARMED;
		status.ejDetect		= FALSE;			// not currently checking for this
		status.ejCont		= EjectionContinuity();
		status.lagged		= TRUE; 			// this is for the lagged buffer
		status.cycleCount8	= CycleCount;
		status.rtc16		= rtcPressure;
		status.rawPressure	= (UINT16) Pressure;
		status.substate		= Substate;
		status.state		= State;

		LogStatusToLaggedBuffer(&status);

		if (DoLogStatus)						// write log entry to flash?
		{
			status.lagged = FALSE;
			WriteToFlashBuffer(&status, sizeof(status));
			DoLogStatus = FALSE;
		}

		if (DoLogFix && FixCount)				// only if we actually ever got a fix
		{
			LogGPSRecord(LastGPSFixPtr);
			DoLogFix = FALSE;
		}

		// idle loop - do while waiting for RTC to tick for LOOP_PERIOD_MS poll

		while (NOTYET8(nextRTC8))
		{
			SET_DEBUG_STATE(CPU_IDLE);

			FlushFlashBufferSafe(1);			// flush to flash only if no GPS burst will be pre-empted

			ServiceBuzzer();
			ServiceHorn();
		}
		SET_DEBUG_STATE(CPU_BUSY);

		nextRTC8 = RTC8 + LOOP_PERIOD;
	}
}



