// (c) 2006 Nerdfever.com

#ifndef __NMEA_GPS_H
#define __NMEA_GPS_H

#define NMEA_BUFSIZE	(83)		// bytes, 80 + CR + LF + null
#define NMEA_LINES		(2)			// NMEA sentance buffers (MUST BE A POWER OF 2)

#define MAX_GOODFIXES	(15)

// misc constants

#define KNOTSperMPS (1.94384449)	// knots per meter/s

#define GPS_BURST_PERIOD		MS(GPS_BURST_PERIOD_MS)

// structures

typedef struct	// GPS fix, full
{
	UINT16		rtc16;				// rtc at time fix BEGAN to be received (1st byte of fix)
	unsigned	satsUsed : 4;		// SVs used for this fix, from GPGGA. If > 15 SVs used, then "15". (LSBs)
	unsigned	fixType  : 2;		// from GPGGA PFI (0=no fix, 1=GPS, 2=Differential GPS)
	unsigned	spareBit : 1;
	unsigned	valid    : 1;		// 1=fix is valid (per GPRMC V/A code) (MSB)
	UINT8		fixMode;			// ASCII code (see below)
	UINT8		year;				// years from 2000 (2008=8)
	UINT8		month;				// month (1=January)
	UINT8		day;				// day of month
	UINT8		hour;				// hour (00 to 23, UTC)
	UINT8		min;				// minute (0 to 59)
	UINT16		milliseconds;		// ms into minute (0 to 59,999)
	INT32		lat10kPerMin;		// units are 1/10000 arc-min from equator
	INT32		lon10kPerMin;		// units are 1/10000 arc-min from Greenwich
	UINT24		groundSpeedcmPerS;	// units are 0.01 m/s
	UINT16		course100PerDeg;	// units are 1/100 degree (0=north, postive values only)
	INT24		altitude10cmMSL;	// units are 0.1 meters above MSL
} GPSFIX;

typedef struct
{
	UINT16		rtc16;				// rtc at time fix BEGAN to be received (1st byte of fix)
	unsigned	fixType  	: 2;	// from GPGGA PFI (0=no fix, 1=GPS, 2=Differential GPS)
	unsigned	valid    	: 1;	// 1=fix is valid (per GPRMC V/A code) (MSB)
	unsigned	numUsedCode	: 2;	// number of satellites used (see code below)
	unsigned	fixModeCode : 3;	// code from fixMode
	INT16		deltaLat10kPerMin;	// 1/10000 arc-min from last logged valid fix (0 to 6079 meters/sample)
	INT16		deltaLon10kPerMin;	// 1/10000 arc-min from last logged valid fix( 0 to 6079 meters/sample)
	UINT16		milliseconds;		// GPS time, milliseconds since start of minute
	UINT16		groundSpeed5cmPerS;	// units are 1/20 meter/sec (range 0 to 3276.75 m/s = 7,330 MPH)
	UINT8		course256PerCircle;	// units are 1/256 of a circle (0 = North)
	UINT16		altCode;			// units are 1/3 meter MSL, - 500 meters (range +21345m to -500m)
} GPSFIX_D;


/* fixMode codes (fixModeCode / ASCII letters):

	0 / *	Field not supplied by GPS (older version of NMEA-0183)
	1 / A	Autonomous fix
	2 / D	Differential fix
	3 / E	Estimated fix (not reliable)
	4 / S	Simulator (not reliable)
	5 / N 	Not valid (not reliable)
	6 / 	reserved for future use
	7 / 	reserved for future use

*/

/*
 numUsedCode
		0	zero satellites used (no fix)
		1	1 to 3 satellites used (2D fix or worse)
		2	4 to 6 satellites used (3D fix)
		3	7 or more satellites used
*/


// function prototypes

void AbsoluteToDifferentialGPSFix(GPSFIX *abs, GPSFIX_D *diff);
void CheckForNewGPSFix(void);
void InitGPS(void);

#endif // __NMEA_GPS_H
