// (c) 2007 Nerdfever.com


#include <math.h>
#include <stdlib.h>		// for ltoa()
#include "hardware.h"
#include "globals.h"
#include "peripherals.h"

// for Freescale MPX4105A pressure sensor (as altimeter)
// returns ISA pressure altitude in feet
// Takes 4.2 ms @ 2 MHz to execute (PIC18)
INT24 PressureToFeet(double pressure)
{
	double 	Vpin;
	double 	kPA;
	double	PAfeet;					// pressure altitude, feet, ISA

	Vpin = Vref_minus + (pressure/PRESSURE_FULLSCALE) * (Vref_plus - Vref_minus);

	kPA = (Vpin/Vdd + 0.09)*100;	// Per Freescale MPX4105A datasheet

	PAfeet = (1.0-pow(kPA/101.325,0.190263))*145440.688060892; // takes 43 MILLISECONDS @ 2 MHz (!!!!)

	return (INT24) PAfeet+0.5;
}

void BeepOutMaxAltitude(void)
{
	INT24 max_altitude_feet_AGL;
	static UINT8 altitude_buffer[9] = "";		// 9 bytes will hold 2^23 plus '-' and null terminator
	static UINT16 next_event_RTC16;
	static UINT8 next_digit_index;
	static UINT8 pulses_left;

	if (!altitude_buffer[0])
	{
		max_altitude_feet_AGL = PressureToFeet(ApogeePressure) - PressureToFeet(PadPressure);

		ltoa(max(max_altitude_feet_AGL,0), (char *) altitude_buffer);

		next_digit_index = 0;
		pulses_left = 0;

		next_event_RTC16 = RTC16;				// start doing something NOW
	}

	if (NOTYET16(next_event_RTC16))				// time for next beep state change event?
		return;

	if (Beep)
	{
		Beep = OFF;
		next_event_RTC16 = RTC16 + BEEP_OFF_PERIOD;

		return;
	}

	if (pulses_left)
	{
		Beep = ON;
		pulses_left--;
		next_event_RTC16 = RTC16 + BEEP_ON_PERIOD;

		return;
	}

	if (altitude_buffer[next_digit_index])
	{
		pulses_left = altitude_buffer[next_digit_index++]-'0';	// convert ASCII digit to binary

		if (pulses_left == '0' - '0')
			pulses_left = 10;									// 10 beeps for a zero

		next_event_RTC16 = RTC16 + INTERDIGIT_PAUSE_PERIOD;

		return;
	}

	next_digit_index = 0;
	next_event_RTC16 = RTC16 + INTERVALUE_PAUSE_PERIOD;
}


// for loud horn
void ServiceHorn(void)
{
	switch (Horn)
	{
		case ON:
			HORN_HW = ON;
			break;

		case OFF:
			HORN_HW = OFF;
			break;

		case FAST_HORN:
			if (RTC16 % ((UINT16)FAST_HORN_ON_PERIOD + FAST_HORN_OFF_PERIOD) > FAST_HORN_ON_PERIOD)
				HORN_HW = OFF;
			else
				HORN_HW = ON;

			break;

		case SLOW_HORN:
			if (RTC16 % (SLOW_HORN_ON_PERIOD + SLOW_HORN_OFF_PERIOD) > SLOW_HORN_ON_PERIOD)
				HORN_HW = OFF;
			else
				HORN_HW = ON;

			break;
	}
}


// check which (if any) switches held down continuously
void PollSwitches(void)
{
	if (SW_RED)
		SwRedHeld += LOOP_PERIOD;			// count how long SW_RED been contiuously held down
	else
		SwRedHeld = 0;

	if (SW_YELLOW)
		SwYellowHeld += LOOP_PERIOD;		// count how long SW_YELLOW been contiuously held down
	else
		SwYellowHeld = 0;

	if (SW_GREEN)
		SwGreenHeld += LOOP_PERIOD;		// count how long SW_GREEN been contiuously held down
	else
		SwGreenHeld = 0;
}

// returns flag TRUE if continuity good, FALSE if bad
UINT8 EjectionContinuity(void)
{
	UINT8 result;

	INTCON2bits.RBPU = ON;			// turn off weak pullups on PORTB
	result = EJ_CONT_OK;			// read value with pullups turned off
	INTCON2bits.RBPU = OFF;			// turn on weak pullups on PORTB

	return result;
}