// (c) 2007 Nerdfever.com

#ifndef __TWEAKS_H
#define __TWEAKS_H

// COMPILE SWITCHES

//#define ESLED_3INCH
//#define ESLED_4INCH_EM406
#define ESLED_4INCH_FGPMMOPA6

	// Platform, E-sled stuff

#ifdef ESLED_4INCH

	#define Vdd				(5.0)			// 7805 regulated
	#define SERVO_MG995						// servo limits - see hardware.h
	#define GPS_EM406		(1)

#endif

#ifdef ESLED_3INCH

	#define Vdd				(3.7)			// LiIon cell
	#define SERVO_MG995						// servo limits - see hardware.h
	#define GPS_GTOP_G33	(2)

#endif

#ifdef ESLED_4INCH_FGPMMOPA6

	#define Vdd				(5.0)			// 7805 regulated
	#define SERVO_MG995						// servo limits - see hardware.h
	#define GPS_FGPMMOPA6	(3)

#endif

// #define COMPASS							// drive Hitachi HM55B compass (Parallax 29123)

// Flash log storage - be careful here not to overwrite program code

#define FLASH_BASE	((UINT24) 0x7800)		// base address of writeable FLASH memory (on row boundary)
#define FLASH_SIZE	((UINT24) 0x8000)		// number of usable bytes (must be a power of 2)

#define FLASH_BASE_POINTER_ADDRESS 0xfd7d	// very last 3 bytes of program ROM
#pragma romdata NORMAL
#pragma romdata overlay FLASH_BASE_POINTER=FLASH_BASE_POINTER_ADDRESS
const static rom UINT24 FLASH_BASE_POINTER = FLASH_BASE;
#pragma romdata NORMAL


// TWEAKABLE SETUP STUFF

#define	FOSC (8000000)						// CPU oscillator frequency, Hz (must be a supported rate in hardware)
#define RTC_INTERRUPT_INTERVAL_MS (1)		// rtc interval (milliseconds)
#define IR_MODULATION_FREQ_HZ (47000)		// infrared "on" pulse modulation frequency (square wave, Hz) (MR-5 = 37 kHz)
#define IR_MODULATION_DUTY_CYCLE (0.4)		// from experiment, works from 0.15 to 0.7 (MR-5 remote = 0.3)

// PRESSURE ALTIMETER RELATED CONSTANTS

#define SLOW_TC					(256)		// "slow" time constant
#define FAST_TC					(4)			// "fast" time constant
#define LAUNCH_THRESHOLD		(44.1)		// "pressure" units (~= -0.4 feet per unit at sea level)
#define TEST_THRESHOLD			(15.)		// "pressure" units, for bench testing

// LOGGING RELATED CONSTANTS

#define LAGGED_RECORDS			(25)		// number of most recent full-rate log records to save upon entering FLIGHT

// time variables in units of RTC ticks

#define SERVO_PULSE_INTERVAL	MS(20)		// Pulse start to pulse start.  Must be multiple of RTC_INTERRUPT_INTERVAL_MS.
#define AUTO_ARM_PERIOD			SECS(60)	// interval between entry into IDLE and auto-entry into ARMED
#define LOOP_PERIOD				MS(40)		// interval for main loop, millisecond (must be multiple of RTC rate)
#define SLOW_LOG_PERIOD			MS(3000)	// interval between log entries in "slow" mode (ARMED state)
#define MED_LOG_PERIOD			MS(200)		// interval between log entries in "medium" mode (FLIGHT DESCENT)
#define SLOW_CAM_PERIOD	 		SECS(20)	// interval between camera frames in "slow" mode
#define FAST_CAM_PERIOD			SECS(3.5)	// interval between camera frames in "fast" mode (FLIGHT)

#define FLIGHT_SAFE_PERIOD		SECS(60)	// "safe" period after pressing FLIGHT switch before effect (for ground testing)
#define EJECT_DURATION			MS(1500)	// duration of ejection trigger pulse
#define HOLD_DOWN_PERIOD		MS(1000)	// duration a button must be held down for secondary action
#define LONG_DOWN_PERIOD		MS(4000)	// duration a button must be held down for tertiary action

#define FAST_HORN_ON_PERIOD		MS(120)
#define FAST_HORN_OFF_PERIOD	MS(160)

#define SLOW_HORN_ON_PERIOD		MS(500)
#define SLOW_HORN_OFF_PERIOD	MS(2000)

#define BEEP_ON_PERIOD 			MS(100)		// beep "on" period
#define BEEP_OFF_PERIOD 		MS(300)		// inter-beep off period
#define INTERDIGIT_PAUSE_PERIOD MS(800)		// inter-digit off period
#define INTERVALUE_PAUSE_PERIOD MS(2000)	// inter-value off period

#ifdef GPS_EM406

	#define GPS_BURST_PERIOD_MS	(60)		// max duration of UART burst per fix (on EM-406 longest seen actual is 44 ms)
	#define GPS_FIX_INTERVAL	MS(1000)	// inverval between GPS updates (EM-406 = 1 Hz)
	#define GPS_TYPE			(GPS_EM406)

#endif

#ifdef GPS_GTOP_G33

	#define GPS_BURST_PERIOD_MS	(20)		// max duration of UART burst per fix (G33 @ 115,200 bps)
	#define GPS_FIX_INTERVAL	MS(200)		// inverval between GPS updates
	#define GPS_TYPE			(GPS_GTOP_G33)

#endif


#ifdef GPS_FGPMMOPA6

	#define GPS_BURST_PERIOD_MS	(70)		// max duration of UART burst per fix (@ 38,400 bps)
	#define GPS_FIX_INTERVAL	MS(200)		// inverval between GPS updates
	#define GPS_TYPE			(GPS_FGPMMOPA6)

#endif

// Debug switches

#ifdef __DEBUG
	#define ICD2							// disable SW_RED (that input does hardware reset on ICD2)
#endif


#endif // __TWEAKS_H